import os
import logging
from datetime import datetime

from ..context import Context
from ..registration import discover_cranium, get_repo_configuration, repo_requires_authentication, \
    get_registration_from_cranium, REPO_AUTH_FILE, load_authentication_details

logger = logging.getLogger(__name__)

INVALID_REGISTRATION = {
    "licensed": False,
    "can_update": False
}

def __call__ (config):
    context = Context(config, logger)
    repo_configuration = get_repo_configuration(context)
    repo_authentication = None

    if not repo_configuration:
        logger.info("No Cattura repository details found for this recorder")
        return INVALID_REGISTRATION

    cattura_auth_file = context.get_path(REPO_AUTH_FILE)
    cattura_repo_host = "repositories.catturavideo.com"

    if os.path.exists(cattura_auth_file):
        auth_details = load_authentication_details(context, cattura_auth_file)
        repo_authentication = auth_details[cattura_repo_host]
    elif "authentication" in repo_configuration:
        repo_authentication = {
            "login": repo_configuration["authentication"][0],
            "password": repo_configuration["authentication"][1]
        }
    else:
        logger.info("Cattura repository details exist for this recorder, but do not contain authentication details")
        return invalid_registration_for_repository(repo_configuration)

    cranium_url = config.registration_url if config.registration_url else repo_configuration["url"]
    cranium = discover_cranium(cranium_url)
    if not cranium:
        logger.warn("Could not detect lemonade stand instance at %s", cranium_url)
        return invalid_registration_for_repository(repo_configuration)
    
    cranium.token = repo_authentication["password"]
    registration = get_registration_from_cranium(cranium, repo_configuration, logger, authentication=repo_authentication)
    log_registration(registration)
    return registration

def invalid_registration_for_repository (repo_configuration):
    registration = {}
    registration.update(INVALID_REGISTRATION)
    registration["can_update"] = not repo_requires_authentication(repo_configuration)
    return registration

def log_registration (registration):
    logger.info("Registration Details:")
    logger.info(" * Licensed to: %s (%s)", registration.licensed_to, registration.email)
    logger.info(" * License key: %s", registration.key)
    logger.info(" * Serial: %s", registration.serial)
    logger.info(" * License valid: %s", registration.valid)
    logger.info(" * Licensed since: %s", format_timestamp(registration.created))
    logger.info(" * Activated on: %s", format_timestamp(registration.activated))
    logger.info(" * Expires on: %s", format_timestamp(registration.expiration_date))
    logger.info(" * Update channel: %s", registration.update_channel)

def format_timestamp (timestamp):
    if timestamp is None:
        return "Never"

    return datetime.fromtimestamp(timestamp / 1000)
